﻿using MigraDoc.DocumentObjectModel;
using MigraDoc.DocumentObjectModel.Shapes;
using MigraDoc.DocumentObjectModel.Tables;
using MigraDoc.Rendering;
using PdfSharp.Pdf;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleApplication2
{
    class Program
    {
        static void Main(string[] args)
        {
            // Create a MigraDoc document
            Document document = CreateDocument();
            document.UseCmykColor = true;

            // ===== Unicode encoding and font program embedding in MigraDoc is demonstrated here =====

            // A flag indicating whether to create a Unicode PDF or a WinAnsi PDF file.
            // This setting applies to all fonts used in the PDF document.
            // This setting has no effect on the RTF renderer.
            const bool unicode = false;

            // An enum indicating whether to embed fonts or not.
            // This setting applies to all font programs used in the document.
            // This setting has no effect on the RTF renderer.
            // (The term 'font program' is used by Adobe for a file containing a font. Technically a 'font file'
            // is a collection of small programs and each program renders the glyph of a character when executed.
            // Using a font in PDFsharp may lead to the embedding of one or more font programms, because each outline
            // (regular, bold, italic, bold+italic, ...) has its own fontprogram)

            // ========================================================================================

            // Create a renderer for the MigraDoc document.
            PdfDocumentRenderer pdfRenderer = new PdfDocumentRenderer(unicode);

            // Associate the MigraDoc document with a renderer
            pdfRenderer.Document = document;

            // Layout and render document to PDF
            pdfRenderer.RenderDocument();

            // Save the document...
            const string filename = "HelloWorld.pdf";
            pdfRenderer.PdfDocument.Save(filename);
            // ...and start a viewer.
            Process.Start(filename);
        }

        /// <summary>
        /// Creates an absolutely minimalistic document.
        /// </summary>
        static Document CreateDocument()
        {
            // Create a new MigraDoc document
            Document document = new Document();

            // Add a section to the document
            Section section = document.AddSection();
            

            // Add a paragraph to the section
            Paragraph paragraph = section.AddParagraph();

            paragraph.Format.Font.Color = Color.FromCmyk(100, 30, 20, 50);

            paragraph.AddFormattedText("Hello, World!", TextFormat.Bold);
            Image image = new Image();


            // Create the item table
            Table table = section.AddTable();
            table.Style = "Table";
            table.Borders.Color = Colors.Gray;
            table.Borders.Width = 0.25;
            table.Borders.Left.Width = 0.5;
            table.Borders.Right.Width = 0.5;
            table.Rows.LeftIndent = 0;

            // Before you can add a row, you must define the columns
            Column column = table.AddColumn("7cm");
            column.Format.Alignment = ParagraphAlignment.Left;

            column = table.AddColumn("7cm");
            column.Format.Alignment = ParagraphAlignment.Right;

            Row row = table.AddRow();
            paragraph = row.Cells[0].AddParagraph();
            paragraph.AddFormattedText("JPG image from file:", TextFormat.Bold);
            paragraph.AddLineBreak();
            image = paragraph.AddImage(@"..\..\jpgimg.jpg");
            image.Width = "5cm";

            paragraph = row.Cells[1].AddParagraph();
            paragraph.AddFormattedText("JPG image from stream:", TextFormat.Bold);
            paragraph.AddLineBreak();
            image = paragraph.AddImage(Base64EncodeFile(@"..\..\jpgimg.jpg"));
            image.Width = "5cm";


            paragraph = row.Cells[0].AddParagraph();
            paragraph.AddFormattedText("PNG image from file:", TextFormat.Bold);
            paragraph.AddLineBreak();
            image = paragraph.AddImage(@"..\..\pngimg.png");
            image.Width = "5cm";

            paragraph = row.Cells[1].AddParagraph();
            paragraph.AddFormattedText("PNG image from stream:", TextFormat.Bold);
            paragraph.AddLineBreak();
            image = paragraph.AddImage(Base64EncodeFile(@"..\..\pngimg.png"));
            image.Width = "5cm";

            return document;
        }

        private static string Base64EncodeFile(string fileName)
        {
            MemoryStream stream = new MemoryStream();
            FileStream fs = File.Open(fileName, FileMode.Open);
            fs.CopyTo(stream);
            string base64 = "base64:" + Convert.ToBase64String(stream.ToArray());
            fs.Close();
            return base64;
        }
    }
}
